function [t,X] = TauLeaping(S,kappa,Sub,dt,X0,T,Tlim)
%TauLeaping Implementation of Euler tau-leaping algorithm
% Inputs:
%   S = stochiometric matrix, n x r
%   kappa = vector of reaction rate constants, 1 x r
%   Sub = reaction substrates matrix, n x r
%   dt = time discretization parameter
%   X0 = initial state, n x 1
%   T = time axis limit
%   Tlim = computational time limit
% Outputs:
%   t = vector of time instants, 1 x Nt
%   X = realized process matrix, n x Nt

% computational time limit
% default value 10 s
if nargin == 6
    Tlim = 10;
end

% start clock
tic

% retrieve lengths
r = length(kappa);

% output variables
t = [];
X = [];

% current state variables
tc = 0;
Xc = X0;

while tc <= T && toc <= Tlim
    
    % add state to output variables
    t = [t tc];
    X = [X Xc];
    
    % compute reaction rates
    lambda = zeros(1,r);
    for k = 1:r
        lambda(k) = ReactionRate(kappa(k),Xc,Sub(:,k));
    end
    
    % sample number of reactions
    numReactions = poissrnd(lambda*dt);
    
    % update state variable
    tc = tc + dt;
    for k = 1:r
        Xc = Xc + numReactions(k)*S(:,k);
    end
    
end

end
